html {
  position: relative;
  min-height: 100%;
}
body {
  /* Margin bottom by footer height */
  margin-bottom: 220px;
}
#inputswrapper {
    min-height:100%;
    /* background: white; */
    position:relative;
    min-width: 800px;
    height: 100%;
    margin: 0 auto;
}
#site-logos {
  margin-top: 1em;
  float: right;
  width: 200px;
}
#site-info {
  float: left;
  margin: 2em auto;
  width: 70%;
}
#info {
  display: inline;
}
#footer {
  position:absolute;
  bottom: 0;
  background:#1F94D3;
  color: white;
  width:100%;
  height:200px;
}
#footer > .container {
  padding-right: 15px;
  padding-left: 15px;
}
#footer a{
  font-weight: bold;
  color: white;
}
#footer :visited{
  font-weight: bold;
  color: black;
}
#form {
  width: 100%;
}
.results {
    overflow: auto;
    /* padding: 20px; */
    background: white;
     /* -moz-border-radius: 20px; */
     /* -webkit-border-radius: 20px; */
     /* -khtml-border-radius: 20px; */
     /* border-radius: 20px; */

}
#input_request { 
    margin-top: 5px;
    display:block;
    word-wrap:break-word;
    white-space:pre;
    overflow: auto;
    margin-top: 20px;
    margin-bottom: 20px;
}
#input_request a{ 
    width:100%;
}


textarea{
    border:1px solid #999999;
    width:100%;
    margin:5px 0;
    padding:3px;
}
.boxsizingBorder {
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  box-sizing: border-box;
}
.valuerow {
  position: relative;
}
.only > .removeclass {
  visibility: hidden;      
}
.key {
  margin: 0.5em !important;
}
.valuerow * {
  vertical-align: middle;
  margin: 0.5em;
  width: 30%;
  padding: 0.5em 1em;
}
.value  {
  margin: 0.5em;
  width: 60%;
  padding: 0.5em 1em;
}
.valuerow input {
  text-align: top;
}
.removeclass {
  padding: 0.5em;
  text-decoration: none;
  color: red;
}
.query
{
  border:1px solid #999999;
  width:100%;
  margin:5px 0;
  padding:3px;
}
#mixedemotions-logo{
  float: right;
  height: 100px;
}
#header-logo{
  height: 50px;
}

.tab-content {
  padding: 1em;
  /* border: 1px solid #ddd; */
  border-top: none;
}
#content-services, #content-resources {
  display: none;
}

#endpoint{
    min-width: 80%;
}

.center-block {
    float:none;
}
#header h3, #header h4, #header h5, #header h6 {
  font-family: 'Architects Daughter', cursive;
}
#header {
    background-color: white;
}

#results-div {
    /* background: white; */
    display: none;
}

.deco-none {
    color: inherit;
    text-decoration: inherit;
}

.deco-none:link {
    color: inherit;
    text-decoration: inherit;
}

.deco-none:hover {
    color: inherit;
    text-decoration: inherit;
}

.collapsed .collapseicon {
    display: none !important;
}

.collapsed .expandicon {
    display: inline-block !important;
}

.expandicon {
    display: none !important;
}

.collapseicon {
    display: inline-block !important;
}

.loader {
    border: 6px solid #f3f3f3; /* Light grey */
    border-top: 6px solid blue;
    border-bottom: 6px solid blue;

    border-radius: 50%;
    width: 3em;
    height: 3em;
    animation: spin 2s linear infinite;

}

small pre {
    font-size: smaller;
    overflow: auto;
}

@keyframes spin {
    0% { transform: rotate(0deg); }
    100% { transform: rotate(360deg); }
}


/* NODES */
.node {
		stroke: #fff;
		fill:#ddd;
		stroke-width: 1.5px;
}
.link {
		fill: none;
		stroke: #999;
		stroke-opacity: .6;
		stroke-width: 1px;
}
marker {
		stroke: #999;
		fill:rgba(124,240,10,0);
}
.node-text {
		font: 11px sans-serif;
		fill:black;
}
.link-text {
		font: 9px sans-serif;
		fill:grey;
}

svg{
		border:1px solid black;
    background-color: white;
    width: 100%;
    height: 600px;
}

.ribbon {
    position: fixed;
    right: 0;
    top: 0;
    z-index: 9999;
}

pre {outline: 1px solid #ccc; padding: 1em; }
.string { color: green; }
.number { color: darkorange; }
.boolean { color: blue; }
.null { color: magenta; }
.key { color: black; }

#basic_params {
    padding: 2em;
}
